package com.viettel.campaign.repository.impl;

import com.viettel.campaign.repository.CampaignRepositoryCustom;
import com.viettel.campaign.utils.Constants;
import com.viettel.campaign.utils.DataUtil;
import com.viettel.campaign.utils.HibernateUtil;
import com.viettel.campaign.utils.SQLBuilder;
import com.viettel.campaign.web.dto.CampaignDTO;
import com.viettel.campaign.web.dto.ResultDTO;
import com.viettel.campaign.web.dto.request_dto.CampaignRequestDTO;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.transform.Transformers;
import org.hibernate.type.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Repository;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.xml.crypto.Data;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Repository
public class CampaignRepositoryImpl implements CampaignRepositoryCustom {
    private static final Logger logger = LogManager.getLogger(CampaignRepositoryImpl.class);

    @Autowired
    EntityManager entityManager;

    @Override
    public List<CampaignDTO> searchCampaignExecute(String agentId, Pageable pageable) {
        List<CampaignDTO> lst = new ArrayList<>();
        String expression = new StringBuilder()
                .append(" SELECT C.CAMPAIGN_ID, C.CAMPAIGN_NAME, C.CONTENT, C.START_TIME, C.END_TIME, C.STATUS ")
                //.append(" SELECT CAMPAIGN_COMPLETE_CODE_ID AS CAMPAIGN_ID, COMPLETE_NAME AS CAMPAIGN_NAME, DESCRIPTION AS CONTENT, UPDATE_BY AS START_TIME, UPDATE_BY AS END_TIME, STATUS ")
                .append(" FROM CAMPAIGN C ")
                .append(" INNER JOIN CAMPAIGN_AGENT CA ON C.CAMPAIGN_ID = CA.CAMPAIGN_ID ")
                .append(" WHERE 1 = 1 ")
                //.append(" AND CA.AGENT_ID = :pAgentId ")
                .append(" AND C.STATUS IN (2,3) ")
                .toString();

        try {
            Query query = entityManager.createNativeQuery(expression);
            //query.setParameter("pAgentId", agentId);

            if (pageable != null) {
                query.setFirstResult(pageable.getPageNumber() * pageable.getPageSize());
                query.setMaxResults(pageable.getPageSize());
            }

            List<Object[]> data = query.getResultList();

            for (Object[] obj : data) {
                CampaignDTO item = new CampaignDTO();
                item.setCampaignId((BigDecimal) obj[0]);
                item.setCampaignName((String) obj[1]);
                item.setContent((String) obj[2]);
                item.setStartTime((Date) obj[3]);
                item.setEndTime((Date) obj[4]);
                item.setStatus(((BigDecimal) obj[5]).shortValueExact());

                lst.add(item);
            }

            return lst;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return lst;
    }

    @Override
    public ResultDTO search(CampaignRequestDTO requestDto) {
        logger.info("Start search campaign::");

        ResultDTO result = new ResultDTO();
        SessionFactory sessionFactory = HibernateUtil.getSessionFactory();

        Session session = sessionFactory.openSession();
        session.beginTransaction();

        if(DataUtil.isNullOrEmpty(requestDto.getCompanySiteId())) {
            result.setErrorCode(Constants.ApiErrorCode.ERROR);
            result.setDescription(Constants.ApiErrorDesc.ERROR);
            return result;
        }
        try {
            StringBuilder sqlStr = new StringBuilder();
            sqlStr.append(SQLBuilder.getSqlQueryById(SQLBuilder.SQL_MODULE_CAMPAIGN_MNG, "search-campaign-by-params"));
            if (!DataUtil.isNullOrEmpty(requestDto.getCampaignCode())) {
                sqlStr.append(" AND a.CAMPAIGN_CODE IN (:p_code) ");
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getCampaignName())) {
                sqlStr.append(" AND UPPER(a.CAMPAIGN_NAME) LIKE :p_name");
            }
            if (requestDto.getStatus() != null && requestDto.getStatus() > 0) {
                sqlStr.append(" AND a.STATUS = :p_status");
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getFromDateFr())) {
                sqlStr.append(" AND TO_DATE(a.START_TIME, 'DD/MM/YYYY') >= :p_frDateFr");
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getToDateFr())) {
                sqlStr.append(" AND TO_DATE(a.START_TIME, 'DD/MM/YYYY') <= :p_toDateFr");
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getFromDateTo())) {
                sqlStr.append(" AND TO_DATE(a.END_TIME, 'DD/MM/YYYY') >= :p_frDateTo");
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getToDateTo())) {
                sqlStr.append(" AND TO_DATE(a.END_TIME, 'DD/MM/YYYY') <= :p_toDateTo");
            }
            if (!DataUtil.isNullOrZero(requestDto.getNumOfCusFr())) {
                sqlStr.append(" AND a.CUSTOMER_NUMBER >= :p_cusNumFr");
            }
            if (!DataUtil.isNullOrZero(requestDto.getNumOfCusTo())) {
                sqlStr.append(" AND a.CUSTOMER_NUMBER <= :p_cusNumTo");
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getTypes())) {
                sqlStr.append(" AND a.CAMPAIGN_TYPE IN (:p_type)");
            }
            if (requestDto.getChanel() != null && !DataUtil.isNullOrZero(requestDto.getChanel())) {
                sqlStr.append(" AND a.CHANEL = :p_chanel");
            }
            if (!DataUtil.isNullOrZero(requestDto.getNumOfCusFr())) {
                sqlStr.append(" AND a.CUSTOMER_NUMBER >= :p_cusNumFr");
            }
            if (!DataUtil.isNullOrZero(requestDto.getNumOfCusTo())) {
                sqlStr.append(" AND a.CUSTOMER_NUMBER <= :p_cusNumTo");
            }

            sqlStr.append(" ORDER BY a.START_TIME DESC ");

            SQLQuery query = session.createSQLQuery(sqlStr.toString());
            if (!DataUtil.isNullOrEmpty(requestDto.getCampaignCode())) {
                String[] lstCode = requestDto.getCampaignCode().split(",");
                query.setParameterList("p_code", lstCode);
            }

            if (!DataUtil.isNullOrEmpty(requestDto.getCampaignName())) {
                query.setParameter("p_name", "%" +
                        requestDto.getCampaignName().toUpperCase()
                                .replace("\\", "\\\\")
                                .replaceAll("%", "\\\\%")
                                .replaceAll("_", "\\\\_")
                        + "%");
            }

            query.setParameter("p_company_site_id", requestDto.getCompanySiteId());

            if (requestDto.getStatus() != null && requestDto.getStatus() > 0) {
                query.setParameter("p_status", requestDto.getStatus());
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getFromDateFr())) {
                query.setParameter("p_frDateFr", requestDto.getFromDateFr());
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getFromDateTo())) {
                query.setParameter("p_frDateTo", requestDto.getFromDateTo());
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getToDateFr())) {
                query.setParameter("p_toDateFr", requestDto.getToDateFr());
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getToDateTo())) {
                query.setParameter("p_toDateTo", requestDto.getToDateTo());
            }
            if (!DataUtil.isNullOrEmpty(requestDto.getTypes())) {
                String[] types = requestDto.getTypes().split(",");
                query.setParameterList("p_type", types);
            }
            if (requestDto.getChanel() != null && !DataUtil.isNullOrZero(requestDto.getChanel())) {
                query.setParameter("p_chanel", requestDto.getChanel());
            }
            if (!DataUtil.isNullOrZero(requestDto.getNumOfCusFr())) {
                query.setParameter("p_cusNumFr", requestDto.getNumOfCusFr() == 0 ? null : requestDto.getNumOfCusFr());
            }
            if (!DataUtil.isNullOrZero(requestDto.getNumOfCusTo())) {
                query.setParameter("p_cusNumTo", requestDto.getNumOfCusTo() == 0 ? null : requestDto.getNumOfCusTo());
            }

            query.addScalar("campaignId", new BigDecimalType());
            query.addScalar("campaignCode", new StringType());
            query.addScalar("campaignName", new StringType());
            query.addScalar("campaignType", new StringType());
            query.addScalar("campaignTypeName", new StringType());
            query.addScalar("chanel", new ShortType());
            query.addScalar("startTime", new DateType());
            query.addScalar("endTime", new DateType());
            query.addScalar("customerNumber", new BigDecimalType());
            query.addScalar("status", new ShortType());
            query.addScalar("numOfJoinedCus", new LongType());
            query.addScalar("numOfNotJoinedCus", new LongType());
            query.addScalar("numOfLockCus", new LongType());

            query.setResultTransformer(Transformers.aliasToBean(CampaignDTO.class));
            int count = 0;
            List<CampaignDTO> list = query.list();
            if (list.size() > 0) {
                count = query.list().size();
            }
            Pageable pageable = buildPageable(requestDto);
            if (pageable != null) {
                query.setFirstResult(pageable.getPageNumber() * pageable.getPageSize());
                query.setMaxResults(pageable.getPageSize());
            }
            List<CampaignDTO> data = query.list();
            Page<CampaignDTO> dataPage = new PageImpl<>(data, pageable, count);
            result.setData(dataPage);
            result.setErrorCode(Constants.ApiErrorCode.SUCCESS);
            result.setDescription(Constants.ApiErrorDesc.SUCCESS);
        } catch (Exception ex) {
            result.setErrorCode(Constants.ApiErrorCode.ERROR);
            result.setDescription(Constants.ApiErrorDesc.ERROR);
            logger.error(ex.getMessage(), ex);
        } finally {
            session.close();
        }
        return result;
    }

    @Override
    public ResultDTO findByCampaignCode(CampaignRequestDTO requestDto) {
        logger.info("Start search campaign by code::");
        ResultDTO result = new ResultDTO();
        SessionFactory sessionFactory = HibernateUtil.getSessionFactory();

        Session session = sessionFactory.openSession();
        session.beginTransaction();

        if(DataUtil.isNullOrEmpty(requestDto.getCompanySiteId())) {
            result.setErrorCode(Constants.ApiErrorCode.ERROR);
            result.setDescription(Constants.ApiErrorDesc.ERROR);
            return result;
        }
        try {
            StringBuilder sqlStr = new StringBuilder();
            sqlStr.append(" SELECT CAMPAIGN_ID campaignId, " +
                    "       CAMPAIGN_CODE campaignCode, " +
                    "       CAMPAIGN_NAME campaignName, " +
                    "       START_TIME startTime," +
                    "       END_TIME endTime, " +
                    "       STATUS status " +
                    " FROM CAMPAIGN" +
                    " WHERE COMPANY_SITE_ID = :p_company_site_id AND STATUS <> -1 ");
            if(!DataUtil.isNullOrEmpty(requestDto.getCampaignCode())) {
                sqlStr.append(" AND CAMPAIGN_CODE LIKE :p_code ");
            }
            sqlStr.append(" ORDER BY START_TIME DESC ");
            SQLQuery query = session.createSQLQuery(sqlStr.toString());
            query.setParameter("p_company_site_id", requestDto.getCompanySiteId());
            if(!DataUtil.isNullOrEmpty(requestDto.getCampaignCode())) {
                query.setParameter("p_code", "%" +
                        requestDto.getCampaignCode().toUpperCase()
                                .replace("\\", "\\\\")
                                .replaceAll("%", "\\\\%")
                                .replaceAll("_", "\\\\_")
                        + "%");
            }
            query.addScalar("campaignId", new BigDecimalType());
            query.addScalar("campaignCode", new StringType());
            query.addScalar("campaignName", new StringType());
            query.addScalar("startTime", new DateType());
            query.addScalar("endTime", new DateType());
            query.addScalar("status", new ShortType());

            query.setResultTransformer(Transformers.aliasToBean(CampaignDTO.class));
            int count = 0;
            List<CampaignDTO> list = query.list();
            if (list.size() > 0) {
                count = query.list().size();
            }
            Pageable pageable = buildPageable(requestDto);
            if (pageable != null) {
                query.setFirstResult(pageable.getPageNumber() * pageable.getPageSize());
                query.setMaxResults(pageable.getPageSize());
            }
            List<CampaignDTO> data = query.list();
            Page<CampaignDTO> dataPage = new PageImpl<>(data, pageable, count);
            result.setData(dataPage);
            result.setErrorCode(Constants.ApiErrorCode.SUCCESS);
            result.setDescription(Constants.ApiErrorDesc.SUCCESS);
        }catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            result.setErrorCode(Constants.ApiErrorCode.ERROR);
            result.setDescription(Constants.ApiErrorDesc.ERROR);
        } finally {
            session.close();
        }
        return result;
    }

    private Pageable buildPageable(CampaignRequestDTO obj) {
        Pageable pageable = null;
        if (DataUtil.isNullOrEmpty(obj.getSort())) {
            pageable = new PageRequest(obj.getPage(), obj.getPageSize(), null);
        } else {
            String[] sorts = obj.getSort().split(",");
            Sort sort = new Sort(Sort.Direction.fromString(sorts[1]), sorts[0]);
            pageable = new PageRequest(obj.getPage(), obj.getPageSize(), sort);
        }
        return pageable;
    }
}
